@@ -23,9 +23,11 @@ import h5py
 import logging
 from pyarrow.dataset import field as ffield
 from pyarrow import dataset as ds
+from typing import Union, Iterable
 from pathlib import Path
 from .tools import bisect_left, bisect_right, to_intdate, read_version_config
 from .config import data_path, cal_config
+
 markets = ["zz500", "hs300", "zz800", "zz1000", "all"]
 logger = logging.getLogger("logger")
 stream_handler = logging.StreamHandler()
@@ -199,12 +201,12 @@ class CalendarProvider:
     def _uri_cal(self):
         return os.path.join(data_path, "calendars", "{}.txt")
 
-    def cache_cal(self, freq):
+    def cache_cal(self, freq: str = "Tday"):
         _calendar = np.array(self.load_calendar(freq))
         _calendar_index = {x: i for i, x in enumerate(_calendar)}  # for fast search
         self.cal_cache[freq] = (_calendar, _calendar_index)
 
-    def load_calendar(self, freq):
+    def load_calendar(self, freq: str = "Tday"):
         """Load original int calendar from file."""
 
         fname = self._uri_cal.format(freq)
@@ -256,7 +258,7 @@ class CalendarProvider:
         end_index = calendar_index[end_time]
         return start_index, end_index
 
-    def _get_calendar(self, freq):
+    def _get_calendar(self, freq: str = "Tday"):
         """Load calendar using memcache."""
         if freq in self.cal_cache:
             return self.cal_cache.get(freq)
@@ -264,21 +266,59 @@ class CalendarProvider:
             self.cache_cal(freq)
             return self.cal_cache.get(freq)
 
-    def predays(self, day, shifts, freq):
+    def cal_shifts(self, dt, shifts, freq: str = "Tday"):
         calendar, calendar_index = self._get_calendar(freq=freq)
-        if shifts > 0:
-            if day not in calendar_index:
-                try:
-                    day = calendar[bisect_left(calendar, day)]
-                except IndexError:
-                    raise IndexError("`start_time` not exist")
-            index = calendar_index[day]
+        if isinstance(dt, Iterable):
+            predt = np.array([self.cal_shifts(dt_i, shifts, freq) for dt_i in dt])
+            return predt
         else:
-            if day not in calendar_index:
-                day = calendar[bisect_right(calendar, day) - 1]
-            index = calendar_index[day]
+            if shifts > 0:
+                index = self.cal_loc(dt, "b", freq)
+                if calendar_index.get(dt) is None:
+                    shifts -= 1
+            else:
+                index = self.cal_loc(dt, "f", freq)
+                if calendar_index.get(dt) is None:
+                    shifts = min(0, shifts + 1)
         return calendar[index + shifts]
 
+    def cal_delta(self, sdt, edt, freq: str = "Tday"):
+        """return diff between dates"""
+        si = self.cal_loc(sdt, "f", freq)
+        ei = self.cal_loc(edt, "f", freq)
+        return ei - si
+
+    def near_dts(self, dt, mode: str = "f", freq: str="Tday"):
+        _calendar, _calendar_index = self._get_calendar(freq=freq)
+        locations = self.cal_loc(dt, mode, freq)
+        return _calendar[locations]
+
+    def cal_loc(self, dt: Iterable, mode: str = "f", freq: str = "Tday"):
+        """
+        :param dt: int or iterable
+        :param mode: f for forward or b for backward if dt not in calendar index
+        :param freq:
+        :return: int or nparray
+        """
+        calendar, calendar_index = self._get_calendar(freq=freq)
+        if isinstance(dt, Iterable):
+            index = np.array([self.cal_loc(dt_i, mode, freq) for dt_i in dt])
+        elif isinstance(dt, int):
+            if dt not in calendar_index:
+                try:
+                    if mode == "f":
+                        dt = calendar[bisect_right(calendar, dt) - 1]
+                    else:
+                        dt = calendar[bisect_left(calendar, dt)]
+                except IndexError:
+                    raise IndexError(
+                        f"`start_time` doesnt exist in {freq} calendar`"
+                    )
+            index = calendar_index[dt]
+        else:
+            raise TypeError("dt is not int or iterable")
+        return index
+
 
 class InstrumentProvider:
     """Instrument provider base class:Provide instrument data."""
